@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009, 2010, 2011, 2012 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +15,11 @@
 
 import argparse
 
+try:
+    import gamera
+except ImportError:
+    gamera = None
+
 from . import djvu_extra as djvu
 from . import version
 
@@ -61,11 +66,19 @@ class intact(object):
     def __call__(self):
         return self.x
 
+def replace_underscores(s):
+    return s.replace('_', '-')
+
 class ArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, methods, default_method):
         argparse.ArgumentParser.__init__(self, formatter_class=argparse.RawDescriptionHelpFormatter)
-        self.add_argument('--version', action='version', version='%(prog)s ' + __version__, help='show version information and exit')
+        version = '%(prog)s ' + __version__
+        try:
+            version += ' (Gamera %s)' % gamera.__version__
+        except (AttributeError, TypeError, ValueError):
+            pass
+        self.add_argument('--version', action='version', version=version, help='show version information and exit')
         p_separate = self.add_subparser('separate', help='generate masks for images')
         p_encode = self.add_subparser('encode', help='convert images to single-page DjVu documents')
         p_bundle = self.add_subparser('bundle', help='convert images to bundled multi-page DjVu document')
@@ -83,7 +96,7 @@ class ArgumentParser(argparse.ArgumentParser):
             p.add_argument('--clean', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_CLEAN, help='lossy compression: remove flyspecks')
             p.add_argument('--lossy', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_LOSSY, help='lossy compression: substitute patterns with small variations')
             if p is not p_separate:
-                p.add_argument('--masks', nargs='+', metavar='MASK', help='use pre-generated masks') 
+                p.add_argument('--masks', nargs='+', metavar='MASK', help='use pre-generated masks')
                 p.add_argument('--mask', action='append', dest='masks', metavar='MASK', help='use a pre-generated mask')
                 for layer, layer_name in ('fg', 'foreground'), ('bg', 'background'):
                     if layer == 'fg':
@@ -97,7 +110,7 @@ class ArgumentParser(argparse.ArgumentParser):
                 p.add_argument('-d', '--dpi', type=dpi_type, metavar='N', help='image resolution')
             if p is p_bundle:
                 p.add_argument('-p', '--pages-per-dict', type=int, metavar='N', help='how many pages to compress in one pass')
-            p.add_argument('-m', '--method', choices=methods, default=default_method, help='binarization method')
+            p.add_argument('-m', '--method', choices=methods, type=replace_underscores, default=default_method, help='binarization method')
             p.add_argument('-v', '--verbose', dest='verbosity', action='append_const', const=None, help='more informational messages')
             p.add_argument('-q', '--quiet', dest='verbosity', action='store_const', const=[], help='no informational messages')
             p.add_argument('input', metavar='<input-image>', nargs='+')
