@@ -1,7 +1,6 @@
-#!/usr/bin/python
 # encoding=UTF-8
 
-# Copyright © 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009, 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +13,10 @@
 
 from __future__ import with_statement
 
+import itertools
 import os
 import re
 import logging
-import string
 import sys
 
 from . import cli
@@ -25,85 +24,11 @@ from . import djvu_extra as djvu
 from . import filetype
 from . import gamera_extra as gamera
 from . import ipc
+from . import templates
 from . import temporary
 
 logger = None
 
-try:
-    from string import Formatter
-except ImportError:
-    # This is very rough, incomplete backport of string.Formatter class.
-    # It's provided here to allow basic functionality of didjvu even with
-    # Python 2.5.
-
-    import warnings
-    warnings.warn(RuntimeWarning('Python %d.%d is not supported, please use Python >= 2.6' % sys.version_info[:2]), stacklevel=999)
-
-    class Formatter():
-
-        _split = re.compile('''
-            ( [{][{] | [}][}] | [{][^}]*[}] )
-        ''', re.VERBOSE).split
-
-        @staticmethod
-        def _not_implemented():
-            raise NotImplementedError('Please use Python >= 2.6')
-
-        def format(self, format_string, *args, **kwargs):
-            return self.vformat(format_string, args, kwargs)
-
-        def vformat(self, format_string, args, kwargs):
-            result = []
-            for literal_text, field_name, format_spec, conversion in self.parse(format_string):
-                if conversion is not None:
-                    self._not_implemented()
-                if literal_text:
-                    result += [literal_text]
-                if field_name is not None:
-                    obj, _ = self.get_field(field_name, args, kwargs)
-                    obj = self.convert_field(obj, conversion)
-                    result += [self.format_field(obj, format_spec)]
-            return ''.join(result)
-
-        def convert_field(self, value, conversion):
-            if conversion is None:
-                return value
-            else:
-                self._not_implemented()
-
-        def get_value(self, field_name, args, kwargs):
-            if args:
-                self._not_implemented()
-            return kwargs[field_name]
-
-        def get_field(self, field_name, args, kwargs):
-            return self.get_value(field_name, args, kwargs), field_name
-
-        def format_field(self, value, format_spec):
-            if format_spec is not None:
-                self._not_implemented()
-            return str(value)
-
-        def parse(self, format_string):
-            for token in self._split(format_string):
-                if token in ('{{', '}}'):
-                    yield token[0], None, None, None
-                elif token[:1] + token[-1:] == '{}':
-                    if (':' in token) or ('!' in token) or ('.' in token):
-                        self._not_implemented()
-                    yield '', token[1:-1], None, None
-                elif token:
-                    yield token, None, None, None
-
-        def get_value(self, key, args, kwargs):
-            if isinstance(key, (int, long)):
-                return args[key]
-            else:
-                return kwargs[key]
-
-formatter = Formatter()
-del Formatter
-
 def setup_logging():
     global logger
     logger = logging.getLogger('didjvu.main')
@@ -123,9 +48,13 @@ def setup_logging():
     handler.setFormatter(formatter)
     ipc_logger.addHandler(handler)
 
+def parallel_for(o, f, *iterables):
+    for args in zip(*iterables):
+        f(o, *args)
+
 def check_tty():
     if sys.stdout.isatty():
-        print >>sys.stderr, 'I won\'t write DjVu data to a terminal.'
+        print >>sys.stderr, 'I won\'t write binary data to a terminal.'
         sys.exit(1)
 
 def copy_file(input_file, output_file):
@@ -203,7 +132,7 @@ def subsample_bg(image, mask, options):
 def make_layer(image, mask, subsampler, options):
     image, mask = subsampler(image, mask, options)
     return djvu.photo_to_djvu(
-        image=image.to_pil(), mask_image=gamera.to_pil_1bpp(mask),
+        image=gamera.to_pil_rgb(image), mask_image=gamera.to_pil_1bpp(mask),
         slices=options.slices, crcb=options.crcb
     )
 
@@ -216,7 +145,7 @@ def image_to_djvu(width, height, image, mask, options):
         loss_level = 0
     sjbz = djvu.bitonal_to_djvu(gamera.to_pil_1bpp(mask), loss_level=loss_level)
     if options.fg_bg_defaults:
-        image = image.to_pil()
+        image = gamera.to_pil_rgb(image)
         return djvu.Multichunk(width, height, dpi, image=image, sjbz=sjbz)
     else:
         fg_djvu = make_layer(image, mask, subsample_fg, options.fg_options)
@@ -234,60 +163,6 @@ def generate_mask(filename, image, method):
     else:
         return gamera.load_image(filename)
 
-def expand_template(template, name, page):
-    '''
-    >>> path = '/path/to/eggs.png'
-    >>> expand_template('{name}', path, 0)
-    '/path/to/eggs.png'
-    >>> expand_template('{base}', path, 0)
-    'eggs.png'
-    >>> expand_template('{name-ext}.djvu', path, 0)
-    '/path/to/eggs.djvu'
-    >>> expand_template('{base-ext}.djvu', path, 0)
-    'eggs.djvu'
-    >>> expand_template('{page}', path, 0)
-    '1'
-    >>> expand_template('{page:04}', path, 0)
-    '0001'
-    >>> expand_template('{page}', path, 42)
-    '43'
-    >>> expand_template('{page+26}', path, 42)
-    '69'
-    >>> expand_template('{page-26}', path, 42)
-    '17'
-    '''
-    base = os.path.basename(name)
-    name_ext, _ = os.path.splitext(name)
-    base_ext, _ = os.path.splitext(base)
-    d = {
-        'name': name, 'name-ext': name_ext,
-        'base': base, 'base-ext': base_ext,
-        'page': page + 1,
-    }
-    for _, var, _, _ in formatter.parse(template):
-        if var is None:
-            continue
-        if '+' in var:
-            sign = +1
-            base_var, offset = var.split('+')
-        elif '-' in var:
-            sign = -1
-            base_var, offset = var.split('-')
-        else:
-            continue
-        try:
-            offset = sign * int(offset, 10)
-        except ValueError:
-            continue
-        try:
-            base_value = d[base_var]
-        except LookupError:
-            continue
-        if not isinstance(base_value, int):
-            continue
-        d[var] = d[base_var] + offset
-    return formatter.vformat(template, (), d)
-
 _pageid_chars = re.compile('^[A-Za-z0-9_+.-]+$').match
 
 def check_pageid_sanity(pageid):
@@ -341,7 +216,7 @@ class main():
         self.check_common(o)
         if o.output is None:
             if o.output_template is not None:
-                o.output = [expand_template(o.output_template, f, n) for n, f in enumerate(o.input)]
+                o.output = [templates.expand(o.output_template, f, n) for n, f in enumerate(o.input)]
             elif len(o.input) == 1:
                 o.output = [sys.stdout]
                 check_tty()
@@ -369,8 +244,7 @@ class main():
 
     def encode(self, o):
         self.check_multi_output(o)
-        for input, mask, output in zip(o.input, o.masks, o.output):
-            self.encode_one(o, input, mask, output)
+        parallel_for(o, self.encode_one, o.input, o.masks, o.output)
 
     def encode_one(self, o, image_filename, mask_filename, output):
         bytes_in = os.path.getsize(image_filename)
@@ -382,7 +256,7 @@ class main():
                 with open(image_filename, 'rb') as djvu_file:
                     copy_file(djvu_file, output)
             else:
-                # TODO: Figure out if how many pages the multi-page document
+                # TODO: Figure out how many pages the multi-page document
                 # consist of. If it's only one, continue.
                 raise NotImplementedError("I don't know what to do with this file")
             return
@@ -408,7 +282,7 @@ class main():
         logger.info('%s:', image_filename)
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
-            # TODO: Figure out if how many pages the document consist of.
+            # TODO: Figure out how many pages the document consist of.
             # If it's only one, extract the existing mask.
             raise NotImplementedError("I don't know what to do with this file")
         logger.info('- reading image')
@@ -431,9 +305,9 @@ class main():
 
     def separate(self, o):
         self.check_multi_output(o)
-        for input, mask, output in zip(o.input, o.masks, o.output):
+        for mask in o.masks:
             assert mask is None
-            self.separate_one(o, input, output)
+        parallel_for(o, self.separate_one, o.input, o.output)
 
     def bundle(self, o):
         self.check_single_output(o)
@@ -442,6 +316,10 @@ class main():
         else:
             self.bundle_complex(o)
 
+    def _bundle_simple_page(self, o, input, mask, component_name):
+        with open(component_name, 'wb') as component:
+            self.encode_one(o, input, mask, component)
+
     def bundle_simple(self, o):
         [output] = o.output
         with temporary.directory() as tmpdir:
@@ -449,12 +327,11 @@ class main():
             component_filenames = []
             for page, (input, mask) in enumerate(zip(o.input, o.masks)):
                 bytes_in += os.path.getsize(input)
-                pageid = expand_template(o.pageid_template, input, page)
+                pageid = templates.expand(o.pageid_template, input, page)
                 # TODO: Check for filename conflicts.
                 check_pageid_sanity(pageid)
                 component_filenames += os.path.join(tmpdir, pageid),
-                with open(component_filenames[-1], 'wb') as component:
-                    self.encode_one(o, input, mask, component)
+            parallel_for(o, self._bundle_simple_page, o.input, o.masks, component_filenames)
             logger.info('bundling')
             djvu_file = djvu.bundle_djvu(*component_filenames)
             try:
@@ -466,36 +343,46 @@ class main():
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in;
         logger.nosy(self.compression_info_template % locals())
 
+    def _bundle_complex_page(self, o, page, minidjvu_in_dir, image_filename, mask_filename, pixels):
+        logger.info('%s:', image_filename)
+        ftype = filetype.check(image_filename)
+        if ftype.like(filetype.djvu):
+            # TODO: Allow to merge existing documents (even multi-page ones).
+            raise NotImplementedError("I don't know what to do with this file")
+        logger.info('- reading image')
+        image = gamera.load_image(image_filename)
+        width, height = image.ncols, image.nrows
+        pixels[0] += width * height
+        logger.nosy('- image size: %d x %d', width, height)
+        mask = generate_mask(mask_filename, image, o.method)
+        logger.info('- converting to DjVu')
+        page.djvu = image_to_djvu(width, height, image, mask, options=o)
+        image = mask = None
+        page.sjbz = djvu.Multichunk(width, height, o.dpi, sjbz=page.djvu['sjbz'])
+        page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.pageid)
+        os.symlink(page.sjbz.save().name, page.sjbz_symlink)
+
     def bundle_complex(self, o):
         [output] = o.output
         with temporary.directory() as minidjvu_in_dir:
             bytes_in = 0
-            pixels = 0
+            pixels = [0]
             page_info = []
             for pageno, (image_filename, mask_filename) in enumerate(zip(o.input, o.masks)):
                 page = namespace()
                 page_info += page,
                 bytes_in += os.path.getsize(image_filename)
-                page.pageid = expand_template(o.pageid_template, image_filename, pageno)
+                page.pageid = templates.expand(o.pageid_template, image_filename, pageno)
                 check_pageid_sanity(page.pageid)
                 # TODO: Check for filename conflicts.
-                logger.info('%s:', image_filename)
-                ftype = filetype.check(image_filename)
-                if ftype.like(filetype.djvu):
-                    # TODO: Allow to merge existing documents (even multi-page ones).
-                    raise NotImplementedError("I don't know what to do with this file")
-                logger.info('- reading image')
-                image = gamera.load_image(image_filename)
-                width, height = image.ncols, image.nrows
-                pixels += width * height
-                logger.nosy('- image size: %d x %d', width, height)
-                mask = generate_mask(mask_filename, image, o.method)
-                logger.info('- converting to DjVu')
-                page.djvu = image_to_djvu(width, height, image, mask, options=o)
-                image = mask = None
-                page.sjbz = djvu.Multichunk(width, height, o.dpi, sjbz=page.djvu['sjbz'])
-                page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.pageid)
-                os.symlink(page.sjbz.save().name, page.sjbz_symlink)
+            parallel_for(o, self._bundle_complex_page,
+                page_info,
+                itertools.repeat(minidjvu_in_dir),
+                o.input,
+                o.masks,
+                itertools.repeat(pixels)
+            )
+            [pixels] = pixels
             with temporary.directory() as minidjvu_out_dir:
                 logger.info('creating shared dictionaries')
                 def chdir():
