@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008, 2009, 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,15 @@ import os
 import re
 import signal
 import subprocess
-import sys
 
-from subprocess import CalledProcessError
-
-logger = logging.getLogger('didjvu.ipc')
+# CalledProcessError, CalledProcessInterrupted
+# ============================================
 
 # Protect from scanadf[0] and possibly other brain-dead software that set
 # SIGCHLD to SIG_IGN.
 # [0] http://bugs.debian.org/596232
-signal.signal(signal.SIGCHLD, signal.SIG_DFL)
+if os.name == 'posix':
+    signal.signal(signal.SIGCHLD, signal.SIG_DFL)
 
 def get_signal_names():
     _signame_pattern = re.compile('^SIG[A-Z0-9]*$')
@@ -46,22 +45,25 @@ def get_signal_names():
         pass
     return dict((no, name) for name, no in data.iteritems())
 
+CalledProcessError = subprocess.CalledProcessError
+
 class CalledProcessInterrupted(CalledProcessError):
 
     _signal_names = get_signal_names()
 
     def __init__(self, signal_id, command):
         Exception.__init__(self, command, signal_id)
+
     def __str__(self):
         signal_name = self._signal_names.get(self.args[1], self.args[1])
-        return 'Command %r was interrputed by signal %s' % (self.args[0], signal_name)
+        return 'Command %r was interrupted by signal %s' % (self.args[0], signal_name)
 
 del get_signal_names
 
-PIPE = subprocess.PIPE
-from subprocess import CalledProcessError
+# Subprocess
+# ==========
 
-def shell_escape(s, safe=re.compile('^[a-zA-Z0-9_+/=.,:-]+$').match):
+def shell_escape(s, safe=re.compile('^[a-zA-Z0-9_+/=.,:%-]+$').match):
     if safe(s):
         return s
     return "'%s'" % s.replace("'", r"'\''")
@@ -78,7 +80,7 @@ class Subprocess(subprocess.Popen):
         env = dict(
                 (k, v)
                 for k, v in env.iteritems()
-                if not (k.startswith('LC_') or k in ('LANG', 'LANGUAGES'))
+                if not (k.startswith('LC_') or k in ('LANG', 'LANGUAGE'))
         )
         if lc_ctype:
             env['LC_CTYPE'] = lc_ctype
@@ -106,6 +108,14 @@ class Subprocess(subprocess.Popen):
         if return_code < 0:
             raise CalledProcessInterrupted(-return_code, self.__command)
 
+# PIPE
+# ====
+
+PIPE = subprocess.PIPE
+
+# Proxy
+# =====
+
 class Proxy(object):
 
     def __init__(self, object, wait_fn, temporaries):
@@ -127,9 +137,18 @@ class Proxy(object):
         self._wait_fn = int
         setattr(self._object, name, value)
 
+# logging support
+# ===============
+
+logger = logging.getLogger('didjvu.ipc')
+
+# __all__
+# =======
+
 __all__ = [
-    'DEBUG',
-    'PIPE', 'Subprocess', 'Proxy',
+    'CalledProcessError', 'CalledProcessInterrupted',
+    'Subprocess', 'PIPE',
+    'Proxy',
 ]
 
 # vim:ts=4 sw=4 et
